/*----------------------

   GATE - Geant4 Application for Tomographic Emission 
   OpenGATE Collaboration 
   OpenGATE Cluster Taskforce

   Jan De Beenhouwer <jan.debeenhouwer@ugent.be> 

   Copyright (C) 2005 ELIS, University of Ghent
   B-9000 Ghent, Belgium  

   Dirk Kruecker extended for seed files 2.6.2005
   JDB added lmf as output format; cleaned up output format checking; 2.8.2005
   JDB resolved bugs in output format handling; splitfile now reports all output 3.8.2005
   S.Stute added Raw output capabilities 29-02-2008

   P.J.Sharrock comented out line 289 output<<"/gate/output/root/setSaveRndmFlag 0"<<endl;
                command not valid in Gate 5.0  05-01-2011
                added command to create log and error directories  07-01-2011
   
This software is distributed under the terms 
of the GNU Lesser General  Public Licence (LGPL) 
See GATE/LICENSE.txt for further details 
----------------------*/

#include "GateMacfileParser.hh"
#include <time.h>

GateMacfileParser::GateMacfileParser(G4String macfileName,G4int numberOfSplits,G4int numberOfAliases,G4String* aliasesPtr)
{
 macName=macfileName;
 nSplits=numberOfSplits;
 nAliases=numberOfAliases;
 aliases=aliasesPtr;
 timeStart=0.0;
 timeStop=0.0;
 lambda=-1;
 usedAliases = new bool[nAliases];
 for(int i=0;i<nAliases;i++)usedAliases[i]=false;

 //root,ascii are enabled by default
 for(int i=2;i<SIZE;i++) enable[i]=0;
 for(int i=2;i<SIZE;i++) filenames[i]=0;
 enable[ROOT] =1;
 enable[ASCII]=1;
 PWD=getenv("PWD");

 // For the random engine's seeds
 srand(time(NULL)*getpid());                                         
}

GateMacfileParser::~GateMacfileParser()
{ 
}

G4int GateMacfileParser::GenerateResolvedMacros(G4String directory)
{
 G4String macNameDir=macName.substr(0,macName.length()-4);
 G4String dir=directory+macNameDir+"/";
 ifstream dirstream(dir.c_str());
 int i=0;
 stringstream i_str; 
 while (dirstream)
 {
  i++;
  i_str.str("");
  i_str<<i;
  dirstream.close();
  dir=directory+macNameDir+i_str.str()+"/";
  dirstream.open(dir.c_str());
 }
 dirstream.close();
 
 const G4String mkdir("mkdir "+dir); 
 const int res = system(mkdir.c_str()); 
 if (res!=0) 
  {
   cout<<"failed to create directory in .Gate"<<endl;
   return 1;
  }
 if(m_verboseLevel>3) cout<<"Information : creating "<<dir<<endl;

 G4String splitfileName=dir+macNameDir+".split";
 ofstream splitfile(splitfileName.c_str());
 splitfile<<"Number of files: "<<nSplits<<endl<<endl; 

 outputDir=dir;

 for (G4int j=1;j<=nSplits;j++)
 {
  i_str.str("");
  i_str<<j;
// Added by PJS to create error and log directories  07-01-2011
  G4String mkdirE("mkdir "+dir+"/err"+i_str.str());     // create error directory
  int Res = system(mkdirE.c_str()); 
  if (Res!=0) 
  {
    cout<<"Failed To Create Error Directory In .Gate"<<endl;
    return 1;
  }
 
  G4String mkdirL("mkdir "+dir+"/log"+i_str.str());     // create error directory
  Res = system(mkdirL.c_str()); 
  if (Res!=0) 
  {
    cout<<"Failed To Create Log Directory In .Gate"<<endl;
    return 1;
  }

  GenerateResolvedMacro(dir+macNameDir+i_str.str()+".mac",j,splitfile); 
  splitfile<<endl;  
 }
 if (enable[ROOT])
   splitfile<<"Original Root filename: "<<originalRootFileName<<endl;
 if (enable[RAW])
   splitfile<<"Original Raw filename: "<<originalRawFileName<<endl; 
 //splitfile<<"Original Interfile filename: "<<originalInterFileName<<endl;
 //splitfile<<"Original ECAT7 filename: "<<originalEcat7FileName<<endl;
 //splitfile<<"Original Lmf1 filename: "<<originalLmf1FileName<<endl;
 //splitfile<<"Original Sinogram filename: "<<originalSinoFileName<<endl;
 //splitfile<<"Original Ascii filename: "<<originalAsciiFileName<<endl;
 splitfile.close();

 outputMacDir=dir+macNameDir;
 return 0; 
}

void GateMacfileParser::CleanAbort(){
 //erase created files and the directory before error exit
 if( outputDir.contains(".Gate/") ){ //just in case to avoid a catastrophe
    const G4String rmfiles("rm -f "+outputDir+"/*"); 
//    const int res = system(rmfiles.c_str()); 
    const G4String rmdir="rm -f -r "+outputDir; 
    const int res2 = system(rmdir.c_str());
    if(res2) {
       G4cout<<"Could not remove "<<outputDir<<endl; 
       G4cout<<"Please remove manually!"<<endl;
    }
 }
}

G4String GateMacfileParser::GetOutputMacDir()
{
 return outputMacDir;
}

G4int GateMacfileParser::GenerateResolvedMacro(G4String outputName,G4int splitNumber,ofstream& splitfile)
{
 char buffer[256];
 ifstream macfile;
 const G4String dir(outputName); 
 ofstream outputMacfile(dir.c_str());
 
 macfile.open(macName);
 if (!macfile || !outputMacfile)
 {  
  cout<< "Error accessing macro input file! "<<macName<< endl; 
  return 1;
 }

 while (!macfile.eof())
 {
 macfile.getline(buffer,256);
 macline=buffer;

 if (!IsComment(macline))
  {
  FormatMacline();
  InsertAliases();
  LookForEnable();
  InsertFileNames(splitNumber,splitfile);
  InsertSubMacros(outputMacfile,splitNumber,splitfile);
  InsertVirtualCommands(outputMacfile,splitNumber,splitfile);
  IgnoreRandomEngineCommand();
  outputMacfile<<macline<<endl; 
  }
 }
 if(splitNumber==1) {
     if(enable[DAQ]==0){
        G4cout<<"This macro file does not contain any startDAQ !"<<G4endl;
        return 1;
     }
     //check if all aliases from the command line are used
     bool flag=true;
     for (G4int i=1;i<nAliases;i+=2) flag&=usedAliases[i];
     if(flag==false) {
        G4cout<<"Could not use the following aliases from the command line:"<<G4endl;
        for (G4int i=1;i<nAliases;i+=2) 
           if(!usedAliases[i]) G4cout<<" "<<aliases[i]<<G4endl;
        return 1; 
     }
  }
 macfile.close();
 outputMacfile.close();
 return 0;
}

void GateMacfileParser::InsertAliases()
{
 G4String insert;
 for (G4int i=1;i<nAliases;i+=2)
  {

   if (macline.contains("{"+aliases[i]+"}"))
    {
     insert=aliases[i-1];
     G4int position=macline.find("{"+aliases[i]+"}",0);
     G4int length=2+aliases[i].size();
     macline.replace(position,length,insert);
     usedAliases[i]=true;
   }
 }
}

void GateMacfileParser::InsertSubMacros(ofstream& output,G4int splitNumber,ofstream& splitfile)
{
 if (macline.contains("/control/execute"))
  {
   char buffer[256];
   G4String extMacfileName=macline.substr(17,256);
   if (extMacfileName.contains("/")) ExtractLocalDirectory(extMacfileName);
   ifstream extMacfile;
   extMacfile.open(extMacfileName);
   if (!extMacfile)
    {  
     G4String localname=localDir+extMacfileName;
     extMacfile.clear();
     extMacfile.open(localname);
     if (!extMacfile)
      { 
       cout<< "Error reading from file: "<<extMacfileName<< endl; 
       CleanAbort();
       exit(1);
      } 
    }
   while (!extMacfile.eof())
    {      
     extMacfile.getline(buffer,256);
     macline=buffer;
     if (!IsComment(macline))
      {
       FormatMacline();
       InsertAliases();
       LookForEnable();
       InsertFileNames(splitNumber,splitfile);
       InsertSubMacros(output,splitNumber,splitfile);
       InsertVirtualCommands(output,splitNumber,splitfile);
       output<<macline<<endl;
      }
    }
   extMacfile.close();
   macline="";
  }
}

void GateMacfileParser::InsertVirtualCommands(ofstream& output,G4int splitNumber,ofstream& splitfile)
{
 if (macline.contains("/gate/cluster/setTimeSplitHalflife"))
 {
  G4String subMacline=macline.substr(35,256);
  G4int position=subMacline.find(" ",0);
  G4String lambda_str=subMacline.substr(0,position);
  G4String temp=lambda_str(position-1);
  if (!strcmp(temp,".")) lambda_str=lambda_str+"0";
  stringstream lambda_ss(lambda_str);
  lambda_ss>>lambda;
  lambda=log(2.0)/lambda;
  macline="";
 }
 if (macline.contains("/gate/application/setTimeStop"))
 {
  G4String subMacline=macline.substr(30,256);
  G4int position=subMacline.find(" ",0);
  G4String timeStop_str=subMacline.substr(0,position);
  G4String temp=timeStop_str(position-1);
  if (!strcmp(temp,".")) timeStop_str=timeStop_str+"0";
  stringstream timeStop_ss(timeStop_str);
  timeStop_ss>>timeStop;
  splitfile<<"Stop time is: "<<macline.substr(30,256)<<endl;
 }

 if (macline.contains("/gate/application/setTimeStart"))
 {
  G4String subMacline=macline.substr(31,256);
  G4int position=subMacline.find(" ",0);
  G4String timeStart_str=subMacline.substr(0,position);
  timeUnit=subMacline.substr(position+1,subMacline.length());
  G4String temp=timeStart_str(position-1);
  if (!strcmp(temp,".")) timeStart_str=timeStart_str+"0";
  stringstream timeStart_ss(timeStart_str);
  timeStart_ss>>timeStart;
  splitfile<<"Start time is: "<<macline.substr(31,256)<<endl;
 }
 if (macline.contains("/gate/application/setTimeSlice"))
 {
  G4String subMacline=macline.substr(31,256);
  splitfile<<"Timeslice is: "<<macline.substr(31,256)<<endl;  
 }

 if (macline.contains("gate/application/startDAQ"))
 {
 
   // to check if there is a DAQ command at all
   enable[DAQ]=1;
   
   //just before we switch off the rndm output
   //output<<"/gate/output/root/setSaveRndmFlag 0"<<endl;  // removed this line PJS 05-01-2011
   if(m_verboseLevel>2) 
     cout<<"Information : random status file output disabled (/gate/output/root/setSaveRndmFlag 0)."<<endl;

   // And we set a brand new seed for the random engine !
   unsigned long int theSeed = rand()*time(NULL);
   output << "/gate/random/setEngineSeed " << theSeed << endl;
 
  // after this command the job will start 
  // so all output should be defined
  if(splitNumber==1) CheckOutputPrint();   
  CheckOutput(output,splitfile,splitNumber);   
   
  // virtual time
  if(timeStop-timeStart==0) {
      G4cout<<"setTimeStart - setTimeStop seems to be 0 ?"<<G4endl;
      CleanAbort();
      exit(1);
  }
  if (lambda!=-1) CalculateTimeSplit(splitNumber);
  else {
        virtualStartTime=timeStart+(timeStop-timeStart)/(G4double)nSplits*(splitNumber-1);
        virtualStopTime=timeStart+(timeStop-timeStart)/(G4double)nSplits*splitNumber;
//        cout<<"Num Splits  "<<nSplits<<endl;
//        cout<<"Split Num  "<<splitNumber<<endl;
//        cout<<"Start Time  "<<timeStart<<endl;
//        cout<<"Stop Time  "<<timeStop<<endl;
//        cout<<"Virtual Start Time  "<<virtualStartTime<<endl;
//        cout<<"Virtual Start Time  "<<virtualStopTime<<endl;
       }
  //output<<"/gate/application/setVirtualStartTime "<<virtualStartTime<<" "<<timeUnit<<endl;
  output<<"/gate/application/startDAQCluster "<<virtualStartTime<<" "<<virtualStopTime<<" "<<"0 "<<timeUnit<<endl; 
  splitfile<<"Virtual starttime: "<<virtualStartTime<<" "<<timeUnit<<endl;
  splitfile<<"Virtual stoptime: "<<virtualStopTime<<" "<<timeUnit<<endl;
  macline="";
 }
}

void GateMacfileParser::IgnoreRandomEngineCommand()
{
  if (macline.contains("/gate/random/setEngineSeed")) {
    macline="";
  }
}

void GateMacfileParser::CalculateTimeSplit(G4int splitNumber)
{
 G4double t1=(log((((G4double)nSplits-1.0)/(G4double)nSplits)*exp(-lambda*timeStart)+(1.0/(G4double)nSplits)*exp(-lambda*timeStop)))/(-lambda);
 if (splitNumber==1)
  { 
   virtualStartTime=timeStart;
   virtualStopTime=t1;
  }
 if (splitNumber==2)
  {
   virtualStartTime=t1;
   virtualStopTime=(log((splitNumber*exp(-lambda*t1))-((splitNumber-1)*exp(-lambda*timeStart))))/(-lambda);
  }
 if (splitNumber>2)
  {
   virtualStartTime=(log(((splitNumber-1)*exp(-lambda*t1))-((splitNumber-2)*exp(-lambda*timeStart))))/(-lambda); 
   if (splitNumber!=nSplits) virtualStopTime=(log((splitNumber*exp(-lambda*t1))-((splitNumber-1)*exp(-lambda*timeStart))))/(-lambda);
   else virtualStopTime=timeStop;
  }
}

void GateMacfileParser::ExtractLocalDirectory(G4String macfileName)
{
 G4String subString;
 G4int size=macfileName.length();
 subString=macfileName.substr(size-1,256);
 G4int i=size;
 while (!subString.contains("/"))
 {
  i--;
  subString=macfileName.substr(i,256);
 }
 localDir=macfileName.substr(0,i+1);
}

/*just note if there is an enable command*/
void GateMacfileParser::LookForEnable(){
//root,ascii,interfile,seed,ecat7,sinogram,lmf,raw
//  0   1       2        3   4      5       6    7
  //ROOT and ASCII are enabled by default, but can be disabled
  if (macline.contains("/gate/output/root/disable"))       enable[ROOT]=0;
  if (macline.contains("/gate/output/root/enable"))        enable[ROOT]=1;
  if (macline.contains("/gate/output/ascii/disable"))      enable[ASCII]=0;
  if (macline.contains("/gate/output/ascii/enable"))       enable[ASCII]=1;
  if (macline.contains("/gate/output/projection/disable")) enable[INTER]=0;
  if (macline.contains("/gate/output/projection/enable"))  enable[INTER]=1;
  if (macline.contains("/gate/output/lmf/disable"))       enable[LMF]=0;
  if (macline.contains("/gate/output/lmf/enable"))        enable[LMF]=1;
  if (macline.contains("/gate/output/raw/setFileName"))    enable[RAW]=1;
  //ECAT7 is auto enabled by choice of ecat system and sinogram output
  if (macline.contains("/gate/output/ecat7/disable"))      enable[ECAT]=5;
  //sinogram is auto enabled by choice of ecat system
  if (macline.contains("/gate/output/sinogram/disable"))   enable[SINO]=5;
}

void GateMacfileParser::CheckOutputPrint()
// all outputs must either be disabled or an unique file name is given
{
 //cases are:
 //1) output disabled and not filename is given -> ok 
 //2) output disabled but a filename is given -> ignored
 //3) output enabled but no filename is given -> corrected by a default name
 //4) output enabled and filename given -> ok, except when the name is invalid (use default again) 
 if(enable[ROOT]==0 && filenames[ROOT]==0) cout<<"ROOT output is disabled"<<endl;
 if(enable[ROOT]==0 && filenames[ROOT]==1) cout<<"ROOT output is disabled but a filename is given anyway???"<<endl; 
 if(enable[ROOT]==1 && filenames[ROOT]==0) cout<<"ROOT output is enabled but no filename is given; using a default one"<<endl;
 if(enable[ROOT]==1 && filenames[ROOT]==1) cout<<"ROOT output is enabled"<<endl;
 if(enable[ASCII]==0 && filenames[ASCII]==0) cout<<"ASCII output is disabled"<<endl;
 if(enable[ASCII]==0 && filenames[ASCII]==1) cout<<"ASCII output is disabled but a filename is given anyway???"<<endl; 
 if(enable[ASCII]==1 && filenames[ASCII]==0) cout<<"ASCII output is enabled but no filename is given; using a default one"<<endl;
 if(enable[ASCII]==1 && filenames[ASCII]==1) cout<<"ASCII output is enabled"<<endl;
 if(enable[INTER]==0 && filenames[INTER]==0) cout<<"INTER output is disabled"<<endl;
 if(enable[INTER]==0 && filenames[INTER]==1) cout<<"INTER output is disabled but a filename is given anyway???"<<endl; 
 if(enable[INTER]==1 && filenames[INTER]==0) cout<<"INTER output is enabled but no filename is given; using a default one"<<endl;
 if(enable[INTER]==1 && filenames[INTER]==1) cout<<"INTER output is enabled"<<endl;
 if(enable[LMF]==0 && filenames[LMF]==0) cout<<"LMF output is disabled"<<endl;
 if(enable[LMF]==0 && filenames[LMF]==1) cout<<"LMF output is disabled but a filename is given anyway???"<<endl; 
 if(enable[LMF]==1 && filenames[LMF]==0) cout<<"LMF output is enabled but no filename is given; using a default one"<<endl;
 if(enable[LMF]==1 && filenames[LMF]==1) cout<<"LMF output is enabled"<<endl;
 if(enable[RAW]==0) cout<<"RAW output is disabled"<<endl;
 if(enable[RAW]==1 && filenames[RAW]==1) cout<<"RAW output is enabled"<<endl;
 if(enable[ECAT]==5 && filenames[ECAT]==0) cout<<"ECAT output is disabled"<<endl;
 if(enable[ECAT]==5 && filenames[ECAT]==1) cout<<"ECAT output is disabled but a filename is given anyway???"<<endl; 
 if(enable[ECAT]==0 && filenames[ECAT]==0) cout<<"ECAT output enabled if ECAT + sinogram selected but no filename is given; using a default one"<<endl;
 if(enable[ECAT]==0 && filenames[ECAT]==1) cout<<"ECAT output is enabled if ECAT + sinogram selected"<<endl;
 if(enable[SINO]==5 && filenames[SINO]==0) cout<<"SINO output is disabled"<<endl;
 if(enable[SINO]==5 && filenames[SINO]==1) cout<<"SINO output is disabled but a filename is given anyway???"<<endl; 
 if(enable[SINO]==0 && filenames[SINO]==0) cout<<"SINO output enabled if ECAT system selected but no filename is given; using a default one"<<endl;
 if(enable[SINO]==0 && filenames[SINO]==1) cout<<"SINO output is enabled if ECAT system selected"<<endl;
 if(enable[SEED]==0) cout<<"Information : Random seed file $(GC_DOT_GATE_DIR)/seed(n).rndm "<<endl;
}

// to avoid problems we add a file name explicitly
void GateMacfileParser::CheckOutput(ofstream& output,ofstream& splitfile,G4int splitNumber) {

  if(enable[ROOT]==1 && filenames[ROOT]==0)
   { 
    output<<"/gate/output/root/setFileName "<<PWD<<"/rootfile"<<splitNumber<<endl;
    splitfile<<"Root filename: "<<PWD<<"/rootfile"<<splitNumber<<endl;
   }
  if(enable[ASCII]==1 && filenames[ASCII]==0)
   {
    output<<"/gate/output/ascii/setFileName "<<PWD<<"/asciifile"<<splitNumber<<endl;
    //splitfile<<"Ascii filename: "<<PWD<<"/asciifile"<<splitNumber<<endl;
   }
  if(enable[INTER]==1 && filenames[INTER]==0)
   {
    output<<"/gate/output/interfile/setFileName "<<PWD<<"/interfile"<<splitNumber<<endl;
    splitfile<<"Interfile filename: "<<PWD<<"/interfile"<<splitNumber<<endl;
   }
  if(enable[LMF]==1   && filenames[LMF]==0)
   {
    output<<"/gate/output/lmf/setFileName "<<PWD<<"/lmffile"<<splitNumber<<endl;
    //splitfile<<"Lmf1 filename: "<<PWD<<"/lmffile"<<splitNumber<<endl;
   }
  if(filenames[SINO]==0)
   {
    //output<<"/gate/output/sinogram/setFileName "<<PWD<<"/sinogramfile"<<splitNumber<<endl;
    //splitfile<<"Sinogram filename: "<<PWD<<"/sinogramfile"<<splitNumber<<endl;
   }
  if(filenames[ECAT]==0)
   {
    //output<<"/gate/output/ecat7/setFileName "<<PWD<<"/ecat7file"<<splitNumber<<endl;
    //splitfile<<"Ecat7 filename: "<<PWD<<"/ecat7file"<<splitNumber<<endl;
   }
}

void GateMacfileParser::AddPWD(G4String key)
{
 G4String filename=ExtractFileName(key);
 long unsigned int pos=filename.rfind("/");
 stringstream ss;
 if(pos==string::npos)
   ss<<key<<" "<<PWD<<"/"<<filename;
 else ss<<key<<" "<<filename;
 macline=ss.str();
}

/*assumes that aliasing has been completed beforehand*/
void GateMacfileParser::InsertFileNames(G4int splitNumber,ofstream& splitfile)
{
 char SplitNumberAsString[10];
 sprintf(SplitNumberAsString,"%i",splitNumber);

 if (TreatOutputStream("/gate/output/interfile/setFileName", "interfile", originalInterFileName, SplitNumberAsString))
    {
     AddPWD("/gate/output/interfile/setFileName");
     splitfile<<"Interfile filename: "<<macline.substr(35,macline.length())<<endl;  
     filenames[INTER]=1;  
    }
 if( TreatOutputStream("/gate/output/root/setFileName", "rootfile", originalRootFileName, SplitNumberAsString) )
    {
     AddPWD("/gate/output/root/setFileName");
     splitfile<<"Root filename: "<<macline.substr(30,macline.length())<<endl;
     filenames[ROOT]=1;
    }
 if( TreatOutputStream("/gate/output/ecat7/setFileName", "ecat7file", originalEcat7FileName, SplitNumberAsString) )
    {
     AddPWD("/gate/output/ecat7/setFileName");  
     splitfile<<"Ecat7 filename: "<<macline.substr(31,macline.length())<<endl;
     filenames[ECAT]=1;
    }
 if( TreatOutputStream("/gate/output/lmf/setFileName", "lmffile", originalLmf1FileName, SplitNumberAsString) )
    {
     AddPWD("/gate/output/lmf/setFileName");  
     //splitfile<<"Lmf1 filename: "<<macline.substr(30,macline.length())<<endl;
     filenames[LMF]=1;
    }
 if( TreatOutputStream("/gate/output/raw/setFileName", "rawfile", originalRawFileName, SplitNumberAsString) )
    {
     AddPWD("/gate/output/raw/setFileName");
     splitfile<<"Raw filename: "<<macline.substr(29,macline.length())<<endl;
     filenames[RAW]=1;
    }
 if( TreatOutputStream("/gate/output/sinogram/setFileName", "sinogramfile", originalSinoFileName, SplitNumberAsString) )
    {
     AddPWD("/gate/output/sinogram/setFileName");  
     //splitfile<<"Sinogram filename: "<<macline.substr(34,macline.length())<<endl;
     filenames[SINO]=1; 
    }
 if( TreatOutputStream("/gate/output/ascii/setFileName", "asciifile", originalAsciiFileName, SplitNumberAsString) )
    {
     AddPWD("/gate/output/ascii/setFileName");
     //splitfile<<"Ascii filename: "<<macline.substr(31,macline.length())<<endl;
     filenames[ASCII]=1;
    }
}

bool GateMacfileParser::IsComment(G4String line)
{
 G4String subString=line(0);
 G4int i=0;
 while (subString.contains(" "))
 {
  i++;
  subString=line(i);
 }
 if (subString.contains("#") || line.empty()) return true;
 else return false;
}

void GateMacfileParser::FormatMacline()
{
 G4int size=0;
 G4String subString;
 G4int position=0;
 //remove any trailing comments
 if (macline.contains("#"))
  {
   G4int position=macline.find("#");
   G4String temp=macline.substr(0,position);
   macline=temp;
  }

 //remove trailing spaces
 size=macline.length();
 subString=macline(size-1);
 while (subString.contains(" "))
 {
  size--;
  subString=macline(size);
 }
 if (size<(int)macline.length())
 {
  subString=macline.substr(0,size+1);
  macline=subString;
 }

 //remove starting spaces

 subString=macline(0);
 size=0;
 while (subString.contains(" "))
 {
  size++;
  subString=macline(size);
 }
 if (size>0)
 {
  subString=macline.substr(size,macline.length());
  macline=subString;
 } 

 //remove intermediate spaces >1
 
 subString=macline;
 while (macline.contains("  "))
 {
  position=macline.find("  ");
  subString=macline.substr(position+1,macline.length());
  macline.replace(position,macline.length(),subString);
 }
}

bool GateMacfileParser::Braced(G4String origFile)
{
 //check for braces in name i.e. unresolved alias
 if(origFile.find("{")!=string::npos&&origFile.find("}")!=string::npos) return true;
 return false;
}

void GateMacfileParser::BraceReplace(G4String def, G4String origFile, char* SplitNumberAsString)
{
 //replace the unresolved alias with default
  if(SplitNumberAsString[0]=='1'&&m_verboseLevel>0)
    cout<<"Warning : alias expected for "<< origFile<<" using default name "<< def <<" and local output !"<<endl;
 macline.replace(macline.length()-origFile.length(),macline.length(),def);
}

const G4String GateMacfileParser::ExtractFileName(G4String key)
{
 //int pos=macline.find(key)+key.length();
 // ugly, is there a simple way to get rid of blanks?
 //char* a=const_cast<char*>( (macline.substr(pos,macline.length())).c_str());
 //return strtok(a," ");
 //this is a simple way ;)
 G4String temp=macline;
 temp.erase(0,key.length()+1);
 return temp;
}

void GateMacfileParser::RemoveExtension()
{   
 //the only valid extension
 long unsigned int pos=macline.rfind(".rndm");//the only valid extension
 if(pos!=string::npos)  {
     Extension=macline.substr(pos,100);
     macline.erase(pos);    
    } 
  else Extension="";
}

bool GateMacfileParser::TreatOutputStream(G4String key, G4String def, G4String& origFile, char* SplitNumberAsString)
{
 if (!macline.contains(key)) return false; // nothing to do
 origFile = ExtractFileName(key); // token after key
 // look for possible unresolved aliases and replace them with default name
 if( Braced(origFile) ) {
   BraceReplace(def,origFile,SplitNumberAsString);
   origFile=def;
   }
 //take care of the .rndm
 RemoveExtension();
 //add split number
 macline+=SplitNumberAsString+Extension;
 return true;
}


